@@ -1,13 +1,10 @@
 /* Please see the LICENSE file for copyright and distribution information */
 
 #include "ruby_libxml.h"
+#include <ruby/io.h>
 
 static ID READ_METHOD;
-#ifdef HAVE_RB_IO_BUFWRITE
-#include <ruby/io.h>
-#else
 static ID WRITE_METHOD;
-#endif /* !HAVE_RB_IO_BUFWRITE */
 
 /* This method is called by libxml when it wants to read
  more data from a stream. We go with the duck typing
@@ -27,25 +24,24 @@ int rxml_read_callback(void *context, char *buffer, int len)
   return (int)size;
 }
 
-int rxml_write_callback(void *context, const char *buffer, int len)
+int rxml_write_callback(VALUE io, const char *buffer, int len)
 {
-#ifndef HAVE_RB_IO_BUFWRITE
-  VALUE io, written, string;
-
-  io = (VALUE) context;
-  string = rb_str_new(buffer, len);
-  written = rb_funcall(io, WRITE_METHOD, 1, string);
-
-  return NUM2INT(written);
-#else
-  return (int)rb_io_bufwrite((VALUE) context, buffer, (size_t)len);
-#endif /* !HAVE_RB_IO_BUFWRITE */
+    if (rb_io_check_io(io) == Qnil)
+    {
+        // Could be StringIO
+        VALUE written, string;
+        string = rb_external_str_new_with_enc(buffer, strlen(buffer), rb_enc_get(io));
+        written = rb_funcall(io, WRITE_METHOD, 1, string);
+        return NUM2INT(written);
+    }
+    else
+    {
+        return (int)rb_io_bufwrite(io, buffer, (size_t)len);
+    }
 }
 
 void rxml_init_io(void)
 {
   READ_METHOD = rb_intern("read");
-#ifndef HAVE_RB_IO_BUFWRITE
   WRITE_METHOD = rb_intern("write");
-#endif /* !HAVE_RB_IO_BUFWRITE */
 }
