@@ -58,8 +58,7 @@ static VALUE rxml_sax_parser_initialize(int argc, VALUE *argv, VALUE self)
 
   if (context == Qnil)
   {
-    rb_warn("Passing no parameters to XML::SaxParser.new is deprecated.  Pass an instance of XML::Parser::Context instead.");
-    context = rb_class_new_instance(0, NULL, cXMLParserContext);
+    rb_raise(rb_eArgError, "An instance of a XML::Parser::Context must be passed to XML::SaxParser.new");
   }
 
   rb_ivar_set(self, CONTEXT_ATTR, context);
@@ -96,9 +95,6 @@ static VALUE rxml_sax_parser_parse(VALUE self)
   /* Now check the parsing result*/
   if (status == -1 || !ctxt->wellFormed)
   {
-    if (ctxt->myDoc)
-      xmlFreeDoc(ctxt->myDoc);
-
     rxml_raise(&ctxt->lastError);
   }
   return Qtrue;
